/* ImportSymbols.rexx - Script which is adding symbols into ACE from:
 *                        - a file containing a list of EQU
 *                        - a listing file generated by vasm
 *                        - a symbols file generated by rasm
 *
 * Syntax: ImportSymbols.rexx <file name> vasm|rasm|equ
 *
 * $VER: ImportSymbols.rexx 1.1 (28.02.17)  2017-2018 by Philippe 'OffseT' Rimauro <offset@cpcscene.net>
 * Requires ACE 1.15 or later
 */

OPTIONS RESULTS
SIGNAL ON BREAK_C

ace_required_version="1.15"
ace_port="ACE.1"

appname="ACE Rexx"
LF="0A"x

PARSE ARG symbol_file file_type

/* Check arguments */
IF symbol_file="" | file_type="" THEN DO
    EXIT FatalError("Error in arguments!","Gasp!")
END
IF file_type~="vasm" & file_type~="rasm" & file_type~="equ" THEN DO
    EXIT FatalError("Only file types vasm and rasm are supported!","Oops!")
END

/* Look for ACE running */
IF ~SHOW("P",ace_port) THEN DO
    EXIT FatalError("ACE CPC Emulator process not found!","Damned!")
END

/* ACE is not the default port */
ADDRESS(ace_port)

/* Check ACE version */
"VERSION"
IF CheckVersion(RESULT,ace_required_version)~=0 THEN DO
    EXIT FatalError("ACE CPC Emulator version" ace_required_version "or better is required!","I will update!")
END

/* Load required external libraries */
IF LoadLib("rexxsupport.library")~=0 THEN DO
    EXIT FatalError("rexxsupport.library not found!*NPlease install it.","Okay!")
END

/* Pause ACE while working */
"PAUSE"

/* Add symbols */
retval = 20;

IF file_type="vasm" THEN DO
    retval = AddSymbols_vasm(symbol_file)
    END
ELSE IF file_type="rasm" THEN DO
    retval = AddSymbols_rasm(symbol_file)
END
ELSE IF file_type="equ" THEN DO
    retval = AddSymbols_equ(symbol_file)
END

IF retval~=0 THEN SAY "Symbols file '"symbol_file"' could not be added!"

/* Start ACE again */
"START"

EXIT

/*
 * Add symbols vasm (providing a listing file generated by vasm)
 */
AddSymbols_vasm:
    PARSE ARG file_name

    IF OPEN(file_handle,file_name,"R")~=0 THEN DO
        DO WHILE ~EOF(file_handle)
            line=READLN(file_handle)
            IF line="Symbols:" THEN DO WHILE ~EOF(file_handle)
                line=READLN(file_handle)
                PARSE UPPER VAR line label " EXPR(" dec_addr "=" hex_addr ")"type
                IF label~="" & dec_addr~="" & hex_addr~="" & type~="INTERNAL" THEN DO
                    /*SAY "Found symbol "label" = &"hex_addr" ("dec_addr")"*/
                    "ADDSYMBOL NAME " label "ADDRESS" hex_addr
                END
            END
        END
        CLOSE(file_handle)
    END
    ELSE RETURN 20

    RETURN 0

/*
 * Add symbols rasm (providing a symbol file generated by rasm)
 */
AddSymbols_rasm:
    PARSE ARG file_name

    IF OPEN(file_handle,file_name,"R")~=0 THEN DO
        DO WHILE ~EOF(file_handle)
            line=READLN(file_handle)
            PARSE VAR line label"#"hex_addr" "bank
            IF label~="" & hex_addr~="" & bank~="" THEN DO
                /*SAY "Found symbol "label" = &"hex_addr*/
                "ADDSYMBOL NAME " label "ADDRESS" "0x"hex_addr
            END
        END
        CLOSE(file_handle)
    END
    ELSE RETURN 20

    RETURN 0

/*
 * Add symbols equ (providing a file containing a list of EQU; suitable for pasmo)
 */
AddSymbols_equ:
    PARSE ARG file_name

    IF OPEN(file_handle,file_name,"R")~=0 THEN DO
        DO WHILE ~EOF(file_handle)
            line=READLN(file_handle)
            PARSE UPPER VAR line label "EQU" "&"hex_addr
            IF label="" THEN DO
                PARSE UPPER VAR line label "EQU" "#"hex_addr
            END
            IF hex_addr="" THEN DO
                PARSE UPPER VAR line label "EQU" "0"hex_addr"H"
            END
            IF hex_addr="" THEN DO
                PARSE UPPER VAR line label "EQU" dec_addr
            END
            IF label~="" THEN DO
                IF hex_addr~="" THEN DO
                    /*SAY "Found symbol "label" = &"hex_addr*/
                    "ADDSYMBOL NAME " label "ADDRESS" "0x"hex_addr
                END
                ELSE IF dec_addr~="" THEN DO
                    /*SAY "Found symbol "label" = "dec_addr*/
                    "ADDSYMBOL NAME " label "ADDRESS" hex_addr
                END
            END
        END
        CLOSE(file_handle)
    END
    ELSE RETURN 20

    RETURN 0

/*
 * Version checker (ver.rev[beta])
 */
CheckVersion:
    PARSE ARG cv "." cr "beta" , rv "." rr "beta"

    IF cv<rv | (cv=rv & cr<rr) THEN DO
        RETURN 5
    END
    RETURN 0

/*
 * External lib loader
 */
LoadLib:
    PARSE ARG lib

    IF SHOW("L",lib)=0 THEN DO
        IF ADDLIB(lib,0,-30,0)=0 THEN DO
            RETURN 20
        END
    END
    RETURN 0

/*
 * Fatal error
 */
FatalError:
    PARSE ARG message,button

    ADDRESS "COMMAND" 'RequestChoice >NIL: TITLE "'||appname||'" BODY "'||message||'" GADGETS "'||button||'"'
    RETURN 20
